{
  "type": "record",
  "name": "DashboardKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "doc": "Key for a Dashboard",
  "fields": [
    {
      "name": "dashboardTool",
      "type": "string",
      "doc": "The name of the dashboard tool such as looker, redash etc.",
      "Searchable": {
        "addToFilters": true,
        "boostScore": 4.0,
        "fieldName": "tool",
        "fieldType": "TEXT_PARTIAL"
      }
    },
    {
      "name": "dashboardId",
      "type": "string",
      "doc": "Unique id for the dashboard. This id should be globally unique for a dashboarding tool even when there are multiple deployments of it. As an example, dashboard URL could be used here for Looker such as 'looker.linkedin.com/dashboards/1234'"
    }
  ],
  "Aspect": {
    "name": "dashboardKey"
  }
}