{
  "type": "record",
  "name": "CorpUserInfo",
  "namespace": "com.linkedin.pegasus2avro.identity",
  "doc": "Linkedin corp user information",
  "fields": [
    {
      "name": "active",
      "type": "boolean",
      "doc": "Whether the corpUser is active, ref: https://iwww.corp.linkedin.com/wiki/cf/display/GTSD/Accessing+Active+Directory+via+LDAP+tools",
      "Searchable": {
        "fieldType": "BOOLEAN",
        "weightsPerFieldValue": {
          "true": 2.0
        }
      }
    },
    {
      "name": "displayName",
      "type": [
        "null",
        "string"
      ],
      "doc": "displayName of this user ,  e.g.  Hang Zhang(DataHQ)",
      "default": null
    },
    {
      "name": "email",
      "type": [
        "null",
        "string"
      ],
      "doc": "email address of this user",
      "default": null,
      "Searchable": {
        "fieldType": "KEYWORD",
        "queryByDefault": true
      }
    },
    {
      "name": "title",
      "type": [
        "null",
        "string"
      ],
      "doc": "title of this user",
      "default": null,
      "Searchable": {
        "fieldType": "KEYWORD",
        "queryByDefault": true
      }
    },
    {
      "name": "managerUrn",
      "type": [
        "null",
        "string"
      ],
      "doc": "direct manager of this user",
      "default": null,
      "Relationship": {
        "entityTypes": [
          "corpUser"
        ],
        "name": "ReportsTo"
      },
      "Searchable": {
        "fieldName": "managerLdap",
        "fieldType": "URN",
        "queryByDefault": true
      },
      "java": {
        "class": "com.linkedin.pegasus2avro.common.urn.CorpuserUrn"
      }
    },
    {
      "name": "departmentId",
      "type": [
        "null",
        "long"
      ],
      "doc": "department id this user belong to",
      "default": null
    },
    {
      "name": "departmentName",
      "type": [
        "null",
        "string"
      ],
      "doc": "department name this user belong to",
      "default": null
    },
    {
      "name": "firstName",
      "type": [
        "null",
        "string"
      ],
      "doc": "first name of this user",
      "default": null
    },
    {
      "name": "lastName",
      "type": [
        "null",
        "string"
      ],
      "doc": "last name of this user",
      "default": null
    },
    {
      "name": "fullName",
      "type": [
        "null",
        "string"
      ],
      "doc": "Common name of this user, format is firstName + lastName (split by a whitespace)",
      "default": null,
      "Searchable": {
        "boostScore": 10.0,
        "fieldType": "TEXT_PARTIAL",
        "queryByDefault": true
      }
    },
    {
      "name": "countryCode",
      "type": [
        "null",
        "string"
      ],
      "doc": "two uppercase letters country code. e.g.  US",
      "default": null
    }
  ],
  "Aspect": {
    "EntityUrns": [
      "com.linkedin.pegasus2avro.common.CorpuserUrn"
    ],
    "name": "corpUserInfo"
  }
}